/*
 *  driver for the second Z8536 CIO on the Eltec
 *	forsyth@caldo.demon.co.uk
 */

#define	ELTECBOTCH	/* see io.h */
#include	"u.h"
#include	"../port/lib.h"
#include	"mem.h"
#include	"dat.h"
#include	"fns.h"
#include	"io.h"
#include	"../port/error.h"

#include	"devtab.h"

#include	"z8536.h"

#define	CIO2A	((CIOdev*)CIO2)

enum {
	Lastreg = 0x2f,
};

enum{
	Qdir,
	Qcio,
	Qa,
	Qb,
	Qc,
	Qctl,
};
Dirtab ciotab[]={
	"cio",		{Qcio},		1,	0600,
	"cioa",		{Qa},		1,	0600,
	"ciob",		{Qb},		1,	0600,
	"cioc",		{Qc},		1,	0600,
	"cioctl",	{Qctl},		0,	0600,
};
#define Nciotab (sizeof(ciotab)/sizeof(Dirtab))

static void
onepointseven(void)
{
	int i;
	for(i = 0; i < 20; i++)
		;
}

#define	CWR(r, v) ciowrreg(dev, (r), (v))
#define	CRD(r)	ciordreg(dev, (r))

static void
ciowrreg(CIOdev *dev, int reg, int val)
{
	dev->ptr = reg;
	onepointseven();
	dev->ptr = val;
}

static int
ciordreg(CIOdev *dev, int reg)
{
	dev->ptr = reg;
	onepointseven();
	return dev->ptr;
}

void
cioreset(void)
{
	CIOdev *dev = CIO2A;
	uchar c;

	/* force CIO to state 0 */
	c = dev->ptr;
	USED(c);
	onepointseven();
	dev->ptr = 0;
	onepointseven();
	c = dev->ptr;
	USED(c);
	onepointseven();

	/* reset */
	CWR(MICR, 1);
	onepointseven();
	dev->ptr = 0;

	CWR(Amode, 0);	/* bit mode */
	CWR(Bmode, 0);
	CWR(Apolarity, 0);
	CWR(Bpolarity, 0);
	CWR(Cpolarity, 0);
	CWR(Adir, AllInput);
	CWR(Bdir, AllInput);
	CWR(Cdir, AllInput & 0xF);
	CWR(Aioctl, NormalIO);
	CWR(Bioctl, NormalIO);
	CWR(Cioctl, NormalIO);
	CWR(MCCR, MasterEnable);
}

void
cioinit(void)
{
}

Chan *
cioattach(char *spec)
{
	return devattach('x', spec);
}

Chan *
cioclone(Chan *c, Chan *nc)
{
	return devclone(c, nc);
}

int
ciowalk(Chan *c, char *name)
{
	return devwalk(c, name, ciotab, (long)Nciotab, devgen);
}

void
ciostat(Chan *c, char *db)
{
	devstat(c, db, ciotab, (long)Nciotab, devgen);
}

Chan *
cioopen(Chan *c, int omode)
{
	if(c->qid.path == CHDIR){
		if(omode != OREAD)
			error(Eperm);
	}
	c->mode = openmode(omode);
	c->flag |= COPEN;
	c->offset = 0;
	return c;
}

void
ciocreate(Chan *c, char *name, int omode, ulong perm)
{
	USED(c, name, omode, perm);
	error(Eperm);
}

void
cioremove(Chan *c)
{
	USED(c);
	error(Eperm);
}

void
ciowstat(Chan *c, char *dp)
{
	USED(c, dp);
	error(Eperm);
}

void
cioclose(Chan *c)
{
	USED(c);
}

long
cioread(Chan *c, void *a, long n, ulong offset)
{
	CIOdev *dev = CIO2A;

	switch((int)(c->qid.path&~CHDIR)){
	case Qdir:
		return devdirread(c, a, n, ciotab, Nciotab, devgen);
	case Qcio:
		if(offset > Lastreg)
			n = 0;
		if(n) {
			*(uchar*)a = CRD(offset);
			n = 1;
		}
		break;
	case Qa:
		if(n && offset==0) {
			*(uchar*)a = dev->a;
			n = 1;
		} else
			n = 0;
		break;
	case Qb:
		if(n && offset==0) {
			*(uchar*)a = dev->b;
			n = 1;
		} else
			n = 0;
		break;
	case Qc:
		if(n && offset==0) {
			*(uchar*)a = dev->c & 0xF;
			n = 1;
		} else
			n = 0;
		break;
	case Qctl:
	default:
		n=0;
		break;
	}
	return n;
}

long
ciowrite(Chan *c, char *a, long n, ulong offset)
{
	CIOdev *dev = CIO2A;
	char buf[32];
	char *field[5];
	int m;

	switch((int)(c->qid.path&~CHDIR)){
	case Qcio:
		if(offset == 0 || offset > Lastreg)
			error(Ebadarg);
		if(n) {
			CWR(offset, *(uchar*)a);
			n = 1;
		}
		break;
	case Qa:
		if(n) {
			dev->a = *(uchar*)a;
			n = 1;
		}
		break;
	case Qb:
		if(n) {
			dev->b = *(uchar*)a;
			n = 1;
		}
		break;
	case Qc:
		if(n) {
			dev->c = *(uchar*)a;
			n = 1;
		}
		break;
	case Qctl:
		if(n > sizeof(buf)-1)
			n = sizeof(buf)-1;
		memmove(buf, a, n);
		buf[n] = 0;
		m = getfields(buf, field, 5, " ");
		if(m < 1)
			error(Ebadarg);
		if(strcmp(field[0], "dir") != 0)
			error(Ebadarg);
		if(m < 3)
			error(Ebadarg);
		if(strcmp(field[1], "a") == 0)
			CWR(Adir, strtoul(field[2], 0, 16));
		else if(strcmp(field[1], "b") == 0)
			CWR(Bdir, strtoul(field[2], 0, 16));
		else if(strcmp(field[1], "c") == 0)
			CWR(Cdir, strtoul(field[2], 0, 16)&0xF);
		else
			error(Ebadarg);
		break;
	default:
		error(Ebadusefd);
	}
	return n;
}
